import json

import os 

try:
    from back.ai_device import AiParams
    from back.ao_device import AoParams
    from back.daq_device import DaqParams
except:
    pass

from pioner.shared.utils import is_int_or_raise, list_bitwise_or
from pioner.shared.constants import *


class JsonReader:
    """Reads a JSON configuration file."""

    def __init__(self, path: str):
        """Initializes dictionary.

        Args:
            path: A string path to JSON file.

        Raises:
            ValueError if file doesn't exist or has invalid extension or is empty.
        """
        self._json = dict()
        if not os.path.exists(path):
            raise ValueError("Settings file doesn't exist.")
        if not os.path.splitext(path)[-1] != JSON_EXTENSION:
            raise ValueError("Settings file doesn't have '{}' extension.".format(JSON_EXTENSION))
        with open(path, 'r') as f:
            self._json = json.load(f)
        if not self._json:
            raise ValueError("Empty settings file defined.")

    def json(self):
        """Provides access to the read dictionary."""
        return self._json

class BackSettings:
    """Parses configuration file with all necessary acquisition parameters."""

    def __init__(self, path: str):
        """Initializes dictionary and checks that all needed fields exist.

        After correct parsing it is possible to obtain DaqParams, AiParams and AoParams.

        Args:
            path: A string path to JSON file.

        Raises:
            ValueError if any field doesn't exist.
        """
        json_dict = JsonReader(path).json()
        if DAQ_SETTINGS_FIELD not in json_dict:
            raise ValueError("No '{}' field found in the settings file.".format(DAQ_SETTINGS_FIELD))
        else:
            self._daq_settings_dict = json_dict[DAQ_SETTINGS_FIELD]
            for field in [DAQ_FIELD, AI_FIELD, AO_FIELD]:
                if field not in self._daq_settings_dict:
                    raise ValueError("No '{}' field found in the settings file.".format(field))
        
        if EXPERIMENT_SETTINGS_FIELD not in json_dict:
            raise ValueError("No '{}' field found in the settings file.".format(EXPERIMENT_SETTINGS_FIELD))
        else:
            self._exp_settings_dict = json_dict[EXPERIMENT_SETTINGS_FIELD]
            for field in [PATHS_FIELD, SCAN_FIELD, MODULATION_FIELD]:
                if field not in self._exp_settings_dict:
                    raise ValueError("No '{}' field found in the settings file.".format(field))

        self._invalid_fields = []
        # only in that order
        self.parse_daq_params()
        self.parse_ai_params()
        self.parse_ao_params()
        self.check_invalid_fields()

    def parse_daq_params(self):
        """Parses all necessary DAQ parameters and fills DaqParams instance."""
        self.daq_params = DaqParams()
        daq_dict = self._daq_settings_dict[DAQ_FIELD]

        if INTERFACE_TYPE_FIELD in daq_dict:
            interface_types = daq_dict[INTERFACE_TYPE_FIELD]
            if isinstance(interface_types, list):
                self.daq_params.interface_type = list_bitwise_or(interface_types)
            elif is_int_or_raise(interface_types):
                self.daq_params.interface_type = int(interface_types)
        else:
            self._invalid_fields.append(INTERFACE_TYPE_FIELD)

        if CONNECTION_CODE_FIELD in daq_dict:
            connection_code = daq_dict[CONNECTION_CODE_FIELD]
            if is_int_or_raise(connection_code):
                self.daq_params.connection_code = int(connection_code)
        else:
            self._invalid_fields.append(CONNECTION_CODE_FIELD)

    def parse_ai_params(self):
        """Parses all necessary analog-input parameters and fills AiParams instance."""
        self.ai_params = AiParams()
        ai_dict = self._daq_settings_dict[AI_FIELD]
        scan_dict = self._exp_settings_dict[SCAN_FIELD]

        if SAMPLE_RATE_FIELD in scan_dict:
            sample_rate = scan_dict[SAMPLE_RATE_FIELD]
            if is_int_or_raise(sample_rate):
                self.ai_params.sample_rate = int(sample_rate)
        else:
            if SAMPLE_RATE_FIELD not in self._invalid_fields:
                self._invalid_fields.append(SAMPLE_RATE_FIELD)

        if RANGE_ID_FIELD in ai_dict:
            range_id = ai_dict[RANGE_ID_FIELD]
            if is_int_or_raise(range_id):
                self.ai_params.range_id = int(range_id)
        else:
            self._invalid_fields.append(RANGE_ID_FIELD)

        if LOW_CHANNEL_FIELD in ai_dict:
            low_channel = ai_dict[LOW_CHANNEL_FIELD]
            if is_int_or_raise(low_channel):
                self.ai_params.low_channel = int(low_channel)
        else:
            self._invalid_fields.append(LOW_CHANNEL_FIELD)

        if HIGH_CHANNEL_FIELD in ai_dict:
            high_channel = ai_dict[HIGH_CHANNEL_FIELD]
            if is_int_or_raise(high_channel):
                self.ai_params.high_channel = int(high_channel)
        else:
            self._invalid_fields.append(HIGH_CHANNEL_FIELD)

        if INPUT_MODE_FIELD in ai_dict:
            input_mode = ai_dict[INPUT_MODE_FIELD]
            if is_int_or_raise(input_mode):
                self.ai_params.input_mode = int(input_mode)
        else:
            self._invalid_fields.append(INPUT_MODE_FIELD)

        if SCAN_FLAGS_FIELD in ai_dict:
            scan_flags = ai_dict[SCAN_FLAGS_FIELD]
            if isinstance(scan_flags, list):
                self.ai_params.scan_flags = list_bitwise_or(scan_flags)
            elif is_int_or_raise(scan_flags):
                self.ai_params.scan_flags = int(scan_flags)
        else:
            self._invalid_fields.append(SCAN_FLAGS_FIELD)

    def parse_ao_params(self):
        """Parses all necessary analog-output parameters and fills AoParams instance."""
        self.ao_params = AoParams()
        ao_dict = self._daq_settings_dict[AO_FIELD]
        scan_dict = self._exp_settings_dict[SCAN_FIELD]

        if SAMPLE_RATE_FIELD in scan_dict:
            sample_rate = scan_dict[SAMPLE_RATE_FIELD]
            if is_int_or_raise(sample_rate):
                self.ao_params.sample_rate = int(sample_rate)
        else:
            if SAMPLE_RATE_FIELD not in self._invalid_fields:
                self._invalid_fields.append(SAMPLE_RATE_FIELD)

        if RANGE_ID_FIELD in ao_dict:
            range_id = ao_dict[RANGE_ID_FIELD]
            if is_int_or_raise(range_id):
                self.ao_params.range_id = int(range_id)
        else:
            self._invalid_fields.append(RANGE_ID_FIELD)

        if LOW_CHANNEL_FIELD in ao_dict:
            low_channel = ao_dict[LOW_CHANNEL_FIELD]
            if is_int_or_raise(low_channel):
                self.ao_params.low_channel = int(low_channel)
        else:
            self._invalid_fields.append(LOW_CHANNEL_FIELD)

        if HIGH_CHANNEL_FIELD in ao_dict:
            high_channel = ao_dict[HIGH_CHANNEL_FIELD]
            if is_int_or_raise(high_channel):
                self.ao_params.high_channel = int(high_channel)
        else:
            self._invalid_fields.append(HIGH_CHANNEL_FIELD)

        if SCAN_FLAGS_FIELD in ao_dict:
            scan_flags = ao_dict[SCAN_FLAGS_FIELD]
            if isinstance(scan_flags, list):
                self.ao_params.scan_flags = list_bitwise_or(scan_flags)
            elif is_int_or_raise(scan_flags):
                self.ao_params.scan_flags = int(scan_flags)
        else:
            self._invalid_fields.append(SCAN_FLAGS_FIELD)

    def check_invalid_fields(self):
        """Raises ValueError if at least one required field is missing in the settings."""
        if self._invalid_fields:
            invalid_fields_str = ", ".join(self._invalid_fields)
            raise ValueError("No fields found in the settings file: {}.".format(invalid_fields_str))

    def get_str(self):
        _daq_params = str(self.daq_params)
        _ai_params = str(self.ai_params)
        _ao_params = str(self.ao_params)
        settings_dict = str({"DAQ":_daq_params, "AI":_ai_params, "AO":_ao_params})
        settings_dict = settings_dict.replace("\'", "\"")               # need because json.loads does not recognie " ' "
        
        return settings_dict

class FrontSettings:
    """Parses configuration file with all necessary acquisition parameters."""

    def __init__(self, path: str):
        """Initializes dictionary and checks that all needed fields exist.

        After correct parsing it is possible to obtain ???? AM: add description.

        Args:
            path: A string path to JSON file.

        Raises:
            ValueError if any field doesn't exist.
        """

        json_dict = JsonReader(path).json()

        if SERVER_SETTINGS_FIELD not in json_dict:
            raise ValueError("No '{}' field found in the settings file.".format(SERVER_SETTINGS_FIELD))
        else:
            self._server_settings_dict = json_dict[SERVER_SETTINGS_FIELD]
            for field in [TANGO_FIELD, HTTP_FIELD]:
                if field not in self._server_settings_dict:
                    raise ValueError("No '{}' field found in the settings file.".format(field))
        
        if EXPERIMENT_SETTINGS_FIELD not in json_dict:
            raise ValueError("No '{}' field found in the settings file.".format(EXPERIMENT_SETTINGS_FIELD))
        else:
            self._exp_settings_dict = json_dict[EXPERIMENT_SETTINGS_FIELD]
            for field in [PATHS_FIELD, SCAN_FIELD, MODULATION_FIELD]:
                if field not in self._exp_settings_dict:
                    raise ValueError("No '{}' field found in the settings file.".format(field))


        self._invalid_fields = []
        # only in that order
        self.parse_server_params()
        self.parse_experiment_params()
        self.check_invalid_fields()
        self.set_server_settings()
        self.set_exp_settings()

    def parse_server_params(self):
        for field in [TANGO_HOST_FIELD, DEVICE_PROXY_FIELD]:
            if field not in self._server_settings_dict[TANGO_FIELD] or \
                not isinstance(self._server_settings_dict[TANGO_FIELD][field], str):
                self._invalid_fields.append(field)

        for field in [HTTP_HOST]:
            if field not in self._server_settings_dict[HTTP_FIELD] or \
                not isinstance(self._server_settings_dict[HTTP_FIELD][field], str):
                self._invalid_fields.append(field)

    def parse_experiment_params(self):
        for field in [CALIB_PATH_FIELD, DATA_PATH_FIELD]:
            if field not in self._exp_settings_dict[PATHS_FIELD] or \
                not isinstance(self._exp_settings_dict[PATHS_FIELD][field], str):
                self._invalid_fields.append(field)

        for field in [SAMPLE_RATE_FIELD]:
            if field not in self._exp_settings_dict[SCAN_FIELD] or \
                not isinstance(self._exp_settings_dict[SCAN_FIELD][field], int):
                self._invalid_fields.append(field)
        
        for field in [FREQUENCY_FIELD, AMPLITUDE_FIELD, OFFSET_FIELD]:
            if field not in self._exp_settings_dict[MODULATION_FIELD] or \
                not isinstance(self._exp_settings_dict[MODULATION_FIELD][field], float):
                self._invalid_fields.append(field)

    def set_server_settings(self):
        if not self._invalid_fields:
            self.tango_host = self._server_settings_dict[TANGO_FIELD][TANGO_HOST_FIELD]
            self.device_proxy = self._server_settings_dict[TANGO_FIELD][DEVICE_PROXY_FIELD]
            self.http_host = self._server_settings_dict[HTTP_FIELD][HTTP_HOST]

    def get_server_settings(self):
        return {TANGO_FIELD: {
                    TANGO_HOST_FIELD: self.tango_host,
                    DEVICE_PROXY_FIELD: self.device_proxy
                    },
                HTTP_FIELD:{
                    HTTP_HOST: self.http_host
                    }
                }

    def set_exp_settings(self):
        if not self._invalid_fields:
            self.calib_path = os.path.abspath(self._exp_settings_dict[PATHS_FIELD][CALIB_PATH_FIELD])
            self.data_path = os.path.abspath(self._exp_settings_dict[PATHS_FIELD][DATA_PATH_FIELD])
            self.sample_rate = self._exp_settings_dict[SCAN_FIELD][SAMPLE_RATE_FIELD]
            self.modulation_frequency = self._exp_settings_dict[MODULATION_FIELD][FREQUENCY_FIELD]
            self.modulation_amplitude = self._exp_settings_dict[MODULATION_FIELD][AMPLITUDE_FIELD]
            self.modulation_offset = self._exp_settings_dict[MODULATION_FIELD][OFFSET_FIELD]

    def get_exp_settings(self):
        return {PATHS_FIELD: {
                    CALIB_PATH_FIELD: self.calib_path,
                    DATA_PATH_FIELD: self.data_path
                    },
                SCAN_FIELD:{
                    SAMPLE_RATE_FIELD: self.sample_rate
                    },
                MODULATION_FIELD: {
                    FREQUENCY_FIELD: self.modulation_frequency,
                    AMPLITUDE_FIELD: self.modulation_amplitude,
                    OFFSET_FIELD: self.modulation_offset
                    },
                }

    def check_invalid_fields(self):
        """Raises ValueError if at least one required field is missing in the settings."""
        if self._invalid_fields:
            invalid_fields_str = ", ".join(self._invalid_fields)
            raise ValueError("Wrong or missing inputs in the settings file: {}.".format(invalid_fields_str))

if __name__ == '__main__':

    _path = SETTINGS_FILE_REL_PATH
    # settings = BackSettings(_path)
    settings = FrontSettings(_path)

    # print(settings.get_str())
    print(settings.data_path)




from silx.gui import qt
import sys

from pioner.front.mainWindow import mainWindow

def pioner_run_ui():
    app = qt.QApplication([])
    sys.excepthook = qt.exceptionHandler
    app.setStyle('Fusion')
    example = mainWindow()
    example.show()
    app.exec()

if __name__ == "__main__":
    pioner_run_ui()


from silx.gui import qt

from pioner.shared.constants import *
from pioner.front.messageWindows import *

class calibWindow(qt.QDialog):
    def __init__(self, parent=None):
        super(calibWindow, self).__init__(parent=parent)

        # ####### UI setup
        # ########################################
        short_line_input_width = 60

        self.setWindowTitle("Calibration info")
        self.setFixedHeight(700)
        self.setFixedWidth(450)
        self.setWindowFlag(qt.Qt.WindowContextHelpButtonHint, False)

        mainLayout = qt.QVBoxLayout()
        mainLayout.setAlignment(qt.Qt.AlignHCenter)
        self.setLayout(mainLayout)
        
        lout_1 = qt.QHBoxLayout()
        mainLayout.addLayout(lout_1)
        labl = qt.QLabel("Calibration info: ")
        labl.setMinimumWidth(75)
        lout_1.addWidget(labl)
        self.calibInfoInput = qt.QLineEdit()
        self.calibInfoInput.setFrame(False)
        lout_1.addWidget(self.calibInfoInput)

        self.ttplBox = qt.QGroupBox("Thermopile temperature")
        mainLayout.addWidget(self.ttplBox)
        lout_0 = qt.QVBoxLayout()
        self.ttplBox.setLayout(lout_0)
        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1)
        self.ttplBoxLabl1 = qt.QLabel("000.000")
        self.ttplBoxLabl2 = qt.QLabel("000.000")
        self.ttplBoxInput1 = qt.QLineEdit()
        self.ttplBoxInput1.setMaximumWidth(short_line_input_width)
        self.ttplBoxInput1.setFrame(False)
        self.ttplBoxResetButton = qt.QPushButton("->0")
        self.ttplBoxResetButton.setFixedWidth(25)
        self.ttplBoxResetButton.setFocusPolicy(qt.Qt.NoFocus)
        lout_1.setSpacing(0)
        lout_1.addWidget(qt.QLabel("Utpl, mV : "))
        lout_1.addWidget(self.ttplBoxLabl1)
        lout_1.addWidget(qt.QLabel(" = "))
        lout_1.addWidget(self.ttplBoxLabl2)
        lout_1.addWidget(qt.QLabel(" + "))
        lout_1.addWidget(self.ttplBoxInput1)
        lout_1.addStretch()
        lout_1.addWidget(self.ttplBoxResetButton)
        lout_2 = qt.QHBoxLayout()
        lout_0.addLayout(lout_2)
        self.ttplBoxLabl2 = qt.QLabel("000.000")
        self.ttplBoxInput2 = qt.QLineEdit()
        self.ttplBoxInput2.setMaximumWidth(short_line_input_width)
        self.ttplBoxInput2.setFrame(False)
        self.ttplBoxInput3 = qt.QLineEdit()
        self.ttplBoxInput3.setMaximumWidth(short_line_input_width)
        self.ttplBoxInput3.setFrame(False)
        lout_2.setSpacing(0)
        lout_2.addWidget(qt.QLabel(u"Ttpl, \u2103 : "))
        lout_2.addWidget(self.ttplBoxLabl2)
        lout_2.addWidget(qt.QLabel(" = "))
        lout_2.addWidget(self.ttplBoxInput2)
        lout_2.addWidget(qt.QLabel(u" \u2219 Utpl + "))
        lout_2.addWidget(self.ttplBoxInput3)
        lout_2.addWidget(qt.QLabel(u" \u2219 "))
        lout_2.addWidget(qt.QLabel("Utpl<sup>2</sup>"))
        lout_2.addStretch()

        self.uhtrBox = qt.QGroupBox("Modulation heater rel. voltage")
        mainLayout.addWidget(self.uhtrBox)
        lout_0 = qt.QHBoxLayout()
        self.uhtrBox.setLayout(lout_0)
        self.uhtrBoxLabl1 = qt.QLabel("000.000")
        self.uhtrBoxLabl2 = qt.QLabel("000.000")
        self.uhtrBoxInput1 = qt.QLineEdit()
        self.uhtrBoxInput1.setMaximumWidth(short_line_input_width)
        self.uhtrBoxInput1.setFrame(False)
        self.uhtrBoxInput2 = qt.QLineEdit()
        self.uhtrBoxInput2.setMaximumWidth(short_line_input_width)
        self.uhtrBoxInput2.setFrame(False)
        self.uhtrBoxResetButton = qt.QPushButton("->0")
        self.uhtrBoxResetButton.setFixedWidth(25)
        self.uhtrBoxResetButton.setFocusPolicy(qt.Qt.NoFocus)
        lout_0.setSpacing(0)
        lout_0.addWidget(qt.QLabel("Uhtr, mV : "))
        lout_0.addWidget(self.uhtrBoxLabl1)
        lout_0.addWidget(qt.QLabel(" = ( "))
        lout_0.addWidget(self.uhtrBoxLabl2)
        lout_0.addWidget(qt.QLabel(" + "))
        lout_0.addWidget(self.uhtrBoxInput1)
        lout_0.addWidget(qt.QLabel(u" ) \u2219 "))
        lout_0.addWidget(self.uhtrBoxInput2)
        lout_0.addStretch()
        lout_0.addWidget(self.uhtrBoxResetButton)
        
        self.ihtrBox = qt.QGroupBox("Modulation heater current")
        mainLayout.addWidget(self.ihtrBox)
        lout_0 = qt.QHBoxLayout()
        self.ihtrBox.setLayout(lout_0)
        self.ihtrBoxLabl1 = qt.QLabel("000.000")
        self.ihtrBoxInput1 = qt.QLineEdit()
        self.ihtrBoxInput1.setMaximumWidth(short_line_input_width)
        self.ihtrBoxInput1.setFrame(False)
        self.ihtrBoxInput2 = qt.QLineEdit()
        self.ihtrBoxInput2.setMaximumWidth(short_line_input_width)
        self.ihtrBoxInput2.setFrame(False)
        self.ihtrBoxLabl2 = qt.QLabel("000.000")
        lout_0.setSpacing(0)
        lout_0.addWidget(qt.QLabel("Ihtr, mA : "))
        lout_0.addWidget(self.ihtrBoxLabl1)
        lout_0.addWidget(qt.QLabel(" = "))
        lout_0.addWidget(self.ihtrBoxInput1)
        lout_0.addWidget(qt.QLabel(" + "))
        lout_0.addWidget(self.ihtrBoxInput2)
        lout_0.addWidget(qt.QLabel(u" \u2219 "))
        lout_0.addWidget(self.ihtrBoxLabl2)
        lout_0.addStretch()
        
        self.thtrBox = qt.QGroupBox("Modulation heater temperature")
        mainLayout.addWidget(self.thtrBox)
        lout_0 = qt.QVBoxLayout()
        self.thtrBox.setLayout(lout_0)
        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1)
        self.thtrBoxLabl1 = qt.QLabel("000.000")
        lout_1.setSpacing(0)
        lout_1.addWidget(qt.QLabel(u"R, \u03A9 : "))
        lout_1.addWidget(self.thtrBoxLabl1)
        lout_1.addStretch()
        lout_2 = qt.QHBoxLayout()
        lout_0.addLayout(lout_2)
        self.thtrBoxLabl2 = qt.QLabel("000.000")
        self.thtrBoxInput1 = qt.QLineEdit()
        self.thtrBoxInput1.setMaximumWidth(short_line_input_width)
        self.thtrBoxInput1.setFrame(False)
        self.thtrBoxInput2 = qt.QLineEdit()
        self.thtrBoxInput2.setMaximumWidth(short_line_input_width)
        self.thtrBoxInput2.setFrame(False)
        self.thtrBoxInput3 = qt.QLineEdit()
        self.thtrBoxInput3.setMaximumWidth(short_line_input_width)
        self.thtrBoxInput3.setFrame(False)
        lout_2.setSpacing(0)
        lout_2.addWidget(qt.QLabel(u"Thtr, \u2103 : "))
        lout_2.addWidget(self.thtrBoxLabl2)
        lout_2.addWidget(qt.QLabel(" = "))
        lout_2.addWidget(self.thtrBoxInput1)
        lout_2.addWidget(qt.QLabel(" + "))
        lout_2.addWidget(self.thtrBoxInput2)
        lout_2.addWidget(qt.QLabel(u" \u2219 R + "))
        lout_2.addWidget(self.thtrBoxInput3)
        lout_2.addWidget(qt.QLabel(u" \u2219 ")) 
        lout_2.addWidget(qt.QLabel("R<sup>2</sup>"))
        lout_2.addStretch()

        self.thtrdBox = qt.QGroupBox("Dynamic modulation heater temperature")
        mainLayout.addWidget(self.thtrdBox)
        lout_0 = qt.QVBoxLayout()
        self.thtrdBox.setLayout(lout_0)
        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1)
        self.thtrdBoxLabl1 = qt.QLabel("000.000")
        self.thtrdBoxLabl2 = qt.QLabel("000.000")
        lout_1.setSpacing(0)
        lout_1.addWidget(qt.QLabel(u"R, \u03A9 : "))
        lout_1.addWidget(self.thtrdBoxLabl1)
        lout_1.addWidget(qt.QLabel(" / "))
        lout_1.addWidget(self.thtrdBoxLabl2)
        lout_1.addStretch()
        lout_2 = qt.QHBoxLayout()
        lout_0.addLayout(lout_2)
        self.thtrdBoxLabl2 = qt.QLabel("000.000")
        self.thtrdBoxInput1 = qt.QLineEdit()
        self.thtrdBoxInput1.setMaximumWidth(short_line_input_width)
        self.thtrdBoxInput1.setFrame(False)
        self.thtrdBoxInput2 = qt.QLineEdit()
        self.thtrdBoxInput2.setMaximumWidth(short_line_input_width)
        self.thtrdBoxInput2.setFrame(False)
        self.thtrdBoxInput3 = qt.QLineEdit()
        self.thtrdBoxInput3.setMaximumWidth(short_line_input_width)
        self.thtrdBoxInput3.setFrame(False)
        lout_2.setSpacing(0)
        lout_2.addWidget(qt.QLabel(u"Thtrd, \u2103 : "))
        lout_2.addWidget(self.thtrdBoxLabl2)
        lout_2.addWidget(qt.QLabel(" = "))
        lout_2.addWidget(self.thtrdBoxInput1)
        lout_2.addWidget(qt.QLabel(" + "))
        lout_2.addWidget(self.thtrdBoxInput2)
        lout_2.addWidget(qt.QLabel(u" \u2219 R + "))
        lout_2.addWidget(self.thtrdBoxInput3)
        lout_2.addWidget(qt.QLabel(u" \u2219 "))
        lout_2.addWidget(qt.QLabel("R<sup>2</sup>"))
        lout_2.addStretch()        
        
        self.theaterBox = qt.QGroupBox("Heater temperature vs heater voltage")
        mainLayout.addWidget(self.theaterBox)
        lout_0 = qt.QVBoxLayout()
        self.theaterBox.setLayout(lout_0)
        lout_1 = qt.QHBoxLayout()
        lout_0.addLayout(lout_1)
        lout_2 = qt.QHBoxLayout()
        lout_0.addLayout(lout_2) 
        self.theaterBoxInput1 = qt.QLineEdit()
        self.theaterBoxInput1.setMaximumWidth(short_line_input_width)
        self.theaterBoxInput1.setFrame(False)
        self.theaterBoxInput2 = qt.QLineEdit()
        self.theaterBoxInput2.setMaximumWidth(short_line_input_width)
        self.theaterBoxInput2.setFrame(False)
        self.theaterBoxInput3 = qt.QLineEdit()
        self.theaterBoxInput3.setMaximumWidth(short_line_input_width)
        self.theaterBoxInput3.setFrame(False)
        self.theaterBoxInput4 = qt.QLineEdit()
        self.theaterBoxInput4.setMaximumWidth(short_line_input_width)
        self.theaterBoxInput4.setFrame(False)
        lout_1.setSpacing(0)
        lout_1.addWidget(qt.QLabel(u"Safe voltage, V = "))
        lout_1.addWidget(self.theaterBoxInput1)
        lout_1.addStretch()
        lout_2.setSpacing(0)
        lout_2.addWidget(qt.QLabel(u"Theater, \u2103 = "))
        lout_2.addWidget(self.theaterBoxInput2)
        lout_2.addWidget(qt.QLabel(u" \u2219 U + "))
        lout_2.addWidget(self.theaterBoxInput3)
        lout_2.addWidget(qt.QLabel(u" \u2219 "))
        lout_2.addWidget(qt.QLabel("U<sup>2</sup> + "))
        lout_2.addWidget(self.theaterBoxInput4)
        lout_2.addWidget(qt.QLabel(u" \u2219 "))
        lout_2.addWidget(qt.QLabel("U<sup>3</sup>"))
        lout_2.addStretch()  

        self.rhtrBox = qt.QGroupBox("Heaters resistance")
        mainLayout.addWidget(self.rhtrBox)
        lout_0 = qt.QHBoxLayout()
        self.rhtrBox.setLayout(lout_0)
        lout_0.setSpacing(0)
        lout_0.addWidget(qt.QLabel(u"R<sub>inner</sub>, \u03A9 = "))
        self.rhtrBoxInput1 = qt.QLineEdit()
        self.rhtrBoxInput1.setMaximumWidth(short_line_input_width)
        self.rhtrBoxInput1.setFrame(False)
        lout_0.addWidget(self.rhtrBoxInput1)
        lout_0.addSpacing(20)
        lout_0.addWidget(qt.QLabel(u"R<sub>guard</sub>, \u03A9 = "))
        self.rhtrBoxInput2 = qt.QLineEdit()
        self.rhtrBoxInput2.setMaximumWidth(short_line_input_width)
        self.rhtrBoxInput2.setFrame(False)
        lout_0.addWidget(self.rhtrBoxInput2)
        lout_0.addStretch()

        self.amplcorBox = qt.QGroupBox("Amplitude correction")
        mainLayout.addWidget(self.amplcorBox)
        lout_0 = qt.QHBoxLayout()
        self.amplcorBox.setLayout(lout_0)
        self.amplcorBoxInput1 = qt.QLineEdit()
        self.amplcorBoxInput1.setMaximumWidth(short_line_input_width)
        self.amplcorBoxInput1.setFrame(False)
        self.amplcorBoxInput2 = qt.QLineEdit()
        self.amplcorBoxInput2.setMaximumWidth(short_line_input_width)
        self.amplcorBoxInput2.setFrame(False)
        self.amplcorBoxInput3 = qt.QLineEdit()
        self.amplcorBoxInput3.setMaximumWidth(short_line_input_width)
        self.amplcorBoxInput3.setFrame(False)
        self.amplcorBoxInput4 = qt.QLineEdit()
        self.amplcorBoxInput4.setMaximumWidth(short_line_input_width)
        self.amplcorBoxInput4.setFrame(False)
        lout_0.addWidget(qt.QLabel(u"Ac = "))
        lout_0.addWidget(self.amplcorBoxInput1)
        lout_0.addWidget(qt.QLabel(" + "))
        lout_0.addWidget(self.amplcorBoxInput2)
        lout_0.addWidget(qt.QLabel(u" \u2219 T + "))
        lout_0.addWidget(self.amplcorBoxInput3)
        lout_0.addWidget(qt.QLabel(u" \u2219 "))
        lout_0.addWidget(qt.QLabel("T<sup>2</sup> + "))
        lout_0.addWidget(self.amplcorBoxInput4)
        lout_0.addWidget(qt.QLabel(u" \u2219 "))
        lout_0.addWidget(qt.QLabel("T<sup>3</sup>"))
        lout_0.addStretch()  
        
        mainLayout.addStretch()
        mainLayout.addStretch()
        hline = qt.QFrame()
        hline.setFrameShape(qt.QFrame.HLine)
        hline.setStyleSheet("color: rgb(220, 220, 220);")
        mainLayout.addWidget(hline)

        lout_1 = qt.QHBoxLayout()
        mainLayout.addLayout(lout_1)
        lout_1.setSpacing(1)
        self.loadCalibButton = qt.QPushButton("Load && Apply")
        lout_1.addWidget(self.loadCalibButton)
        self.saveCalibButton = qt.QPushButton("Save && Apply")
        lout_1.addWidget(self.saveCalibButton)
        self.resetCalibButton = qt.QPushButton("Reset")
        self.resetCalibButton.setFocusPolicy(qt.Qt.NoFocus)
        lout_1.addWidget(self.resetCalibButton)


        float_validator = qt.QRegExpValidator(qt.QRegExp("^[-]{0,1}[0-9]{1,5}\.[0-9]{1,10}$|^[-]{0,1}[0-9]{1,5}\.[0-9]{1,10}e[-]{0,1}[+]{0,1}[0-9]{0,2}$"))
        self.update_calib_input_fields()
        for item in self.findChildren(qt.QPushButton):
            item.setFocusPolicy(qt.Qt.NoFocus)
        for item in self.findChildren(qt.QLineEdit):
            item.setAlignment(qt.Qt.AlignCenter)
            item.setCursorPosition(0)
            if item != self.calibInfoInput:
                item.setValidator(float_validator)
                
        # ####### end of UI setup
        # ########################################
        
        self.loadCalibButton.clicked.connect(self.load_calib_from_file)
        self.resetCalibButton.clicked.connect(self.reset_calib)
        self.saveCalibButton.clicked.connect(self.save_calib_to_file)

    def load_calib_from_file(self):
        self.parent().select_calibration_file()
        self.parent().apply_calib()
        self.update_calib_input_fields()

    def save_calib_to_file(self, fpath=False):
        fpath = qt.QFileDialog.getSaveFileName(self, "Select file and path to save calibration:", None, "*.json")[0]
        if fpath:
            self.read_calib_input_fields()
            print(self.parent().calibration.comment)
            self.parent().calibPathInput.setText(os.path.abspath(fpath))
            self.parent().settings.calib_path = fpath
            self.parent().calibPathInput.setCursorPosition(0)
            # self.parent().apply_calib()
            # URL = self.parent().settings.http_host+"settings/calibration.json"
            # response = requests.get(URL, verify=False)
            # with open(fpath, 'wb') as f:
            #     f.write(response.content)

    def reset_calib(self):
        self.parent().apply_default_calib()

    def update_calib_input_fields(self):     
        self.calibInfoInput.setText(str(self.parent().calibration.comment))
        self.ttplBoxInput1.setText(str(round(self.parent().calibration.utpl0, 10)))
        self.ttplBoxInput2.setText(str(round(self.parent().calibration.ttpl0, 10)))
        self.ttplBoxInput3.setText(str(round(self.parent().calibration.ttpl1, 10)))

        self.uhtrBoxInput1.setText(str(round(self.parent().calibration.uhtr0, 10)))
        self.uhtrBoxInput2.setText(str(round(self.parent().calibration.uhtr1, 10)))

        self.ihtrBoxInput1.setText(str(round(self.parent().calibration.ihtr0, 10)))
        self.ihtrBoxInput2.setText(str(round(self.parent().calibration.ihtr1, 10)))

        self.thtrBoxInput1.setText(str(round(self.parent().calibration.thtr0, 10)))
        self.thtrBoxInput2.setText(str(round(self.parent().calibration.thtr1, 10)))
        self.thtrBoxInput3.setText(str(round(self.parent().calibration.thtr2, 10)))

        self.thtrdBoxInput1.setText(str(round(self.parent().calibration.thtrd0, 10)))
        self.thtrdBoxInput2.setText(str(round(self.parent().calibration.thtrd1, 10)))
        self.thtrdBoxInput3.setText(str(round(self.parent().calibration.thtrd2, 10)))

        self.theaterBoxInput1.setText(str(round(self.parent().calibration.safe_voltage, 10)))
        self.theaterBoxInput2.setText(str(round(self.parent().calibration.theater0, 10)))
        self.theaterBoxInput3.setText(str(round(self.parent().calibration.theater1, 10)))
        self.theaterBoxInput4.setText(str(round(self.parent().calibration.theater2, 10)))

        self.rhtrBoxInput1.setText(str(round(self.parent().calibration.rhtr, 10)))
        self.rhtrBoxInput2.setText(str(round(self.parent().calibration.rghtr, 10)))

        self.amplcorBoxInput1.setText(str(round(self.parent().calibration.ac0, 10)))
        self.amplcorBoxInput2.setText(str(round(self.parent().calibration.ac1, 10)))
        self.amplcorBoxInput3.setText(str(round(self.parent().calibration.ac2, 10)))
        self.amplcorBoxInput4.setText(str(round(self.parent().calibration.ac2, 10)))


    def read_calib_input_fields(self):  
        self.parent().calibration.comment = self.calibInfoInput.text()

if __name__ == "__main__":
    import sys
    app = qt.QApplication(sys.argv)
    example = calibWindow()
    example.show()
    sys.exit(app.exec())


from bisect import bisect_left
from typing import List

import numpy as np

from pioner.shared.calibration import Calibration


def is_int(key) -> bool:
    if isinstance(key, int) or isinstance(key, str) and key.isdigit():
        return True


def is_int_or_raise(key) -> bool:
    if is_int(key):
        return True
    raise ValueError("'{}' is not an integer value.".format(key))


def list_bitwise_or(ints: List[int]) -> int:
    res = 0
    for i in ints:
        res |= i
    return res

# calorimeter utils
# ====================================================
def voltage_to_temperature(voltage: np.array, calibration: Calibration) -> np.array:
    volt = voltage.copy()
    volt[volt < 0] = 0
    volt[volt > calibration.safe_voltage] = calibration.safe_voltage
    temp = calibration.theater0 * volt + calibration.theater1 * (volt**2) + calibration.theater2 * (volt**3)
    return temp


# TODO: think maybe to create a T-V (and vice versa) converter class
def temperature_to_voltage(temp: np.array, calibration:  Calibration) -> np.array:
    # generating temp-volt dependency in full calibration range
    resolution = 0.0001  # V
    volt_calib = np.linspace(0, calibration.safe_voltage, int(1 / resolution))

    temp_calib = voltage_to_temperature(volt_calib, calibration)
    temp_calib[temp_calib <= calibration.min_temp] = calibration.min_temp
    temp_calib[temp_calib >= calibration.max_temp] = calibration.max_temp

    voltage = np.zeros(len(temp))

    for i, t in np.ndenumerate(temp):
        idx = bisect_left(temp_calib, t)
        voltage[i] = volt_calib[idx]

    return voltage.round(4)


if __name__ == '__main__':

    # import matplotlib.pyplot as plt
    from time import time

    temp_exp_1 = np.zeros(1000) - 1
    temp_exp_2 = np.linspace(-1, 300, 3000)
    temp_exp_3 = np.ones(1000) + 299
    temp_exp_4 = np.linspace(300, -2, 3000)
    temp_exp_5 = np.zeros(1000) - 2
    temp_exp = np.concatenate((temp_exp_1, temp_exp_2, temp_exp_3, temp_exp_4, temp_exp_5))
    # plt.plot(temp_exp)
    # plt.show()

    t1 = time()
    volt_exp = temperature_to_voltage(temp_exp, Calibration())
    t2 = time()
    print(t2 - t1)
    print(volt_exp[1000:1100])
    # plt.plot(temp_exp, label = 'temp_exp')
    # plt.plot(volt_exp, label = 'volt_exp')
    # plt.legend()
    # plt.show()